<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
		xmlns:xlink="http://www.w3.org/1999/xlink"
    xmlns:d="http://docbook.org/ns/docbook">
  
  <xsl:import href="../../docbook-xsl/xhtml/chunk.xsl"/>
  <xsl:import href="../../docbook-xsl/xhtml/highlight.xsl"/>
  <xsl:import href="../../docbook-xsl/webhelp/xsl/webhelp-common.xsl"/>
  <xsl:import href="../../docbook-xsl/webhelp/xsl/titlepage.templates.xsl"/>
  
  <xsl:import href="../Common/common.xsl"/>
  <xsl:import href="../../Preprocess/Xsl/slideReference.xsl"/>
  
  <xsl:template match="/">    

    <xsl:for-each select="//d:chapter[contains(@annotations, 'slide')]|
                          //d:section[contains(@annotations, 'slide')]">
      <xsl:call-template name="slideUnit">
        <xsl:with-param name="rootId" select="@xml:id"/>
        <xsl:with-param name="sections" select="."/>
      </xsl:call-template>
    </xsl:for-each>
    

    <xsl:for-each select="//d:chapter[descendant::*[contains(@annotations, 'slide')]]|
                          //d:section[descendant::*[contains(@annotations, 'slide')]]">
      <xsl:call-template name="slideUnit">
        <xsl:with-param name="rootId" select="@xml:id"/>
        <xsl:with-param name="sections" select="descendant::*[contains(@annotations, 'slide')]"/>
      </xsl:call-template>
    </xsl:for-each>
    
  </xsl:template>
  
  <xsl:template name="slideUnit">
    <xsl:param name="rootId"/>
    <xsl:param name="sections"/>
    
    <xsl:variable name="slideFileName">
      <xsl:call-template name="slideFilename">
        <xsl:with-param name="slideContainerId" select="$rootId"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:document href="{$slideFileName}" method="html" encoding="utf-8" indent="no">
        
      <html lang="en">
        <head>
          <title>
            <xsl:value-of select="d:title"/>
          </title>
          <link href="common/css/positioning.css" rel="stylesheet" />
          <link href="reveal.js/css/reveal.css" rel="stylesheet" />
          <link href="reveal.js/css/theme/serif.css" id="theme" rel="stylesheet" />
          
          <!-- Code syntax highlighting -->
          <link href="reveal.js/lib/css/zenburn.css" rel="stylesheet" />
          
        </head>
        
        <body>
          <!-- Wrap the entire slide show in a div using the "reveal" class. -->
          <div class="reveal">
            <!-- Wrap all slides in a single "slides" class -->
            <div class="slides">
              <xsl:apply-templates select="$sections" mode="slideDescendants"/>
            </div>
          </div>
          
          <script src="reveal.js/lib/js/head.min.js"></script>
          <script src="reveal.js/js/reveal.js"></script>
          <script>/*<![CDATA[*/
            
            Reveal.initialize({
            controls: true,
            progress: true,
            history: true,
            center: true,
            width: '70%',
	          margin: 0.0,
            
            transition: 'convex', // none/fade/slide/convex/concave/zoom
            
            math: {
            mathjax: 'mathjax/MathJax.js',
            config: 'TeX-AMS-MML_HTMLorMML',  // See http://docs.mathjax.org/en/latest/config-files.html
            },
            // Optional reveal.js plugins
            dependencies: [
            { src: 'reveal.js/lib/js/classList.js', condition: function() { return !document.body.classList; } },
            { src: 'reveal.js/plugin/markdown/marked.js', condition: function() { return !!document.querySelector( '[data-markdown]' ); } },
            { src: 'reveal.js/plugin/markdown/markdown.js', condition: function() { return !!document.querySelector( '[data-markdown]' ); } },
            { src: 'reveal.js/plugin/highlight/highlight.js', async: true, callback: function() { hljs.initHighlightingOnLoad(); } },
            { src: 'reveal.js/plugin/zoom-js/zoom.js', async: true },
            { src: 'reveal.js/plugin/notes/notes.js', async: true },
            { src: 'reveal.js/plugin/math/math.js', async: true }
            ]
            });
            /*]]>*/</script>
        </body>
      </html>
    </xsl:document>
  </xsl:template>
  
  <xsl:template match="d:chapter|d:section" mode="slideDescendants">
    <xsl:apply-templates select="d:section|d:figure"  mode="slideDescendants"/>
  </xsl:template>
  
  <xsl:template match="d:title" mode="slideDescendants"/>
  
  <xsl:template match="d:figure" mode="slideDescendants">
    
    <xsl:variable name="primarySectionId">
      <xsl:choose>
        <xsl:when test="@xml:id"><xsl:value-of select="@xml:id"/></xsl:when>
        <xsl:otherwise><xsl:value-of select="generate-id(.)"/></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <section id="{$primarySectionId}">
            
      <xsl:apply-templates select="."/>
      <a style="font-size: 50%" target="_blank">
        <xsl:attribute name="href">
          <xsl:call-template name="href.target.uri"/>
        </xsl:attribute>
        <xsl:text>Lecture notes</xsl:text>
      </a>
    </section>

    <xsl:variable name="currentFigureId" select="generate-id()"/>
    
    <xsl:variable name="followupQandasets" 
      select="following::d:qandaset[generate-id(preceding::d:figure[1]) = $currentFigureId]"/>
    
    <xsl:if test="$followupQandasets">
      
      <xsl:if test="10 &lt; count($followupQandasets)">
        <xsl:message>
          <xsl:text>Warning: </xsl:text>
          <xsl:value-of select="count($followupQandasets)"/>
          <xsl:text> consecutive followup exercises at slide '</xsl:text>
          <xsl:apply-templates select="d:title"/>
          <xsl:text>', id=</xsl:text>
          <xsl:value-of select="@xml:id"/>
        </xsl:message>
      </xsl:if>
      <section id="{$primarySectionId}_followupExercise">
        <xsl:choose>
          <xsl:when test="1 = count($followupQandasets)">
            <h3>Followup exercise</h3>
            <xsl:apply-templates select="$followupQandasets" mode="createTitleLink"/>
          </xsl:when>
          <xsl:otherwise>
            <h3>Followup exercises</h3>
            <ul>
              <xsl:for-each select="$followupQandasets">
                <li>
                  <xsl:apply-templates select="." mode="createTitleLink"/>
                </li>                
              </xsl:for-each>
            </ul>
          </xsl:otherwise>
        </xsl:choose>
        
      </section>
      
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="d:qandaset" mode="createTitleLink">
    <a target="_blank">
      <xsl:attribute name="href">
        <xsl:call-template name="href.target.uri"/>
      </xsl:attribute>
      <xsl:choose>
        <xsl:when test="d:title">
          <xsl:copy-of select="d:title/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="preceding-sibling::d:title[1]/node()"/>
        </xsl:otherwise>
      </xsl:choose>
    </a>
  </xsl:template>
  
  <xsl:template match="*" mode="createTitleLink">
    <p>
      <xsl:text>No template defined for element '</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>' in mode 'createTitleLink'.</xsl:text>
    </p>
  </xsl:template>
  
  <xsl:template match="*" mode="slideDescendants">
    <p>
      <xsl:text>No template defined for element '</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>' in mode 'slideDescendants'.</xsl:text>
    </p>
  </xsl:template>
  
  <!-- Get rid of block object names e.g. "Figure 3.1" -->
  <xsl:template match="d:figure">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="d:figure/d:title">
    <h3>
      <xsl:apply-templates/>
    </h3>
  </xsl:template>
  
  <!-- Hack: Docbook XSL does not handle @valign in images-->  
  <xsl:template match="d:mediaobject[d:imageobject/d:imagedata[@valign]]">
    <img>
      <xsl:attribute name="src">
        <xsl:if test="ancestor::*/@xml:base">
          <xsl:call-template name="hdm.get.leftmost.of.separator">
            <xsl:with-param name="path" select="ancestor::*/@xml:base"/>
            <xsl:with-param name="separator" select="'/'"/>
          </xsl:call-template>
          <xsl:text>/</xsl:text>
        </xsl:if>
        <xsl:value-of select="d:imageobject/d:imagedata/@fileref"/>
      </xsl:attribute>
      
      
      <xsl:attribute name="style">
        <xsl:variable name="dbValign" select="d:imageobject/d:imagedata/@valign"/>
        
        <xsl:text>vertical-align: </xsl:text>
        <xsl:choose>
          <xsl:when test="$dbValign = 'top'">text-top</xsl:when>
          <xsl:when test="$dbValign = 'middle'">middle</xsl:when>
          <xsl:when test="$dbValign = 'bottom'">text-bottom</xsl:when>
          <xsl:otherwise>
            <xsl:message>
              <xsl:text>Unknown @valign value: </xsl:text>
              <xsl:value-of select="$dbValign"/>
            </xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
    </img>
    
  </xsl:template>
  
  <xsl:template name="hdm.get.leftmost.of.separator">
    <xsl:param name="path"/>
    <xsl:param name="separator"/>
    <xsl:param name="start">1</xsl:param>
    <xsl:if test="contains($path, $separator )">
      <xsl:if test="$start='0'">
        <xsl:value-of select="$separator"/>
      </xsl:if>
      <xsl:value-of select="substring-before($path, $separator)"/>
      <xsl:call-template name="hdm.get.leftmost.of.separator">
        <xsl:with-param name="path" select="substring-after($path, $separator)"/>
        <xsl:with-param name="separator" select="$separator"/>
        <xsl:with-param name="start">0</xsl:with-param>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  
  
</xsl:stylesheet>
